require 'uri'
require 'open-uri'
require "happymapper"
require 'tuffnells_api/tracking_record'

module TuffnellsApi
  class Base
    include HappyMapper

    attr_reader :account_code

    def initialize consignment_no
      @consignment_no = consignment_no
      @account_code = TuffnellsApi.configuration.account_code
      read_timeout = TuffnellsApi.configuration.read_timeout || 5

      proxy = TuffnellsApi.configuration.proxy

      request = Nokogiri::XML(open(base_uri,
        proxy_http_basic_authentication: proxy,
        read_timeout: read_timeout
      ))

      @response = TuffnellsApi::TrackingRecord.parse(request, single: true)
    end

    def postcode
      @response.delivery_address.postcode
    end

    def total_pieces
      @response.consignment_information.pieces
    end

    def delivery_status
      @response.movement_information.movements.last.description
    end

    def signed_by
      @response.timed_information.deliveries.last.signature
    end

    def pod_image
      @response.image_information.images.last.uri
    end

    private

    def base_uri
      "http://www.tpeweb.co.uk/WebServices/Customer/PODTrackingData.asmx/GetTrackingRecord?#{query_string}"
    end

    def query_string
      "AccountCode=#{@account_code}&Reference=#{@consignment_no}"
    end
  end
end
